@@ -24,6 +24,9 @@ void XPS_boot_parse_subsignature_ex(pTHX);
 #define signature_add_param(ctx, details)  XPS_signature_add_param(aTHX_ ctx, details)
 void XPS_signature_add_param(pTHX_ struct XSParseSublikeContext *ctx, struct XPSSignatureParamDetails *details);
 
+#define signature_query(ctx, q)  XPS_signature_query(aTHX_ ctx, q)
+IV XPS_signature_query(pTHX_ struct XSParseSublikeContext *ctx, int q);
+
 #define parse_subsignature_ex(flags, ctx, hd, nhooks)  XPS_parse_subsignature_ex(aTHX_ flags, ctx, hd, nhooks)
 OP *XPS_parse_subsignature_ex(pTHX_ int flags,
   struct XPSContextWithPointer *ctx,
